'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _q = require('q');

var _q2 = _interopRequireDefault(_q);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _recompose = require('recompose');

var _reactResolver = require('react-resolver');

var _fluxibleAddonsReact = require('fluxible-addons-react');

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var fluxibleContextTypes = {
    executeAction: _react.PropTypes.func.isRequired,
    getStore: _react.PropTypes.func.isRequired
};

var formatList = function formatList(list) {
    return list.map(function (e) {
        return '"' + e + '"';
    }).join(', ');
};

var assertValidResolverKeys = function assertValidResolverKeys(displayName, resolvers) {
    (0, _assert2.default)((0, _lodash.isPlainObject)(resolvers), displayName + ' - resolvers must be an object not a ' + (typeof resolvers === 'undefined' ? 'undefined' : _typeof(resolvers)));
    (0, _lodash.every)(resolvers, function (resolver, name) {
        (0, _assert2.default)((0, _lodash.isPlainObject)(resolver), displayName + ' - resolver "' + name + '" must be an object');
    });
    (0, _lodash.every)(resolvers, function (_ref, name) {
        var dependencies = _ref.dependencies;
        var isResolved = _ref.isResolved;
        var resolver = _ref.resolver;

        var extraneous = _objectWithoutProperties(_ref, ['dependencies', 'isResolved', 'resolver']);

        var extraneousKeys = Object.keys(extraneous);
        _assert2.default.equal(extraneousKeys.length, 0, displayName + ' - resolver "' + name + '" unknown option(s) ' + formatList(extraneousKeys));
        (0, _assert2.default)((0, _lodash.isFunction)(resolver), displayName + ' - resolver must be a function');
        if (!(0, _lodash.isUndefined)(dependencies)) {
            (0, _assert2.default)((0, _lodash.isArray)(dependencies), displayName + ' - dependencies must be an array');
            (0, _assert2.default)((0, _lodash.every)(dependencies, _lodash.isString), displayName + ' - dependencies must be strings');
        }
        if (!(0, _lodash.isUndefined)(isResolved)) {
            (0, _assert2.default)((0, _lodash.isFunction)(isResolved), displayName + ' - isResolved must be a function');
        }
    });
};

var assertValidResolverDependencies = function assertValidResolverDependencies(displayName, resolvers) {
    return (0, _lodash.every)(resolvers, function (_ref2, propName) {
        var _ref2$dependencies = _ref2.dependencies;
        var dependencies = _ref2$dependencies === undefined ? [] : _ref2$dependencies;
        return (0, _lodash.every)(dependencies, function (dependency) {
            return (0, _assert2.default)(resolvers.hasOwnProperty(dependency), displayName + ' - "' + propName + '" depends on "' + dependency + '", which does not exist');
        });
    });
};

var assertNoResolversPropTypesCollisions = function assertNoResolversPropTypesCollisions(displayName, resolvers, propTypes) {
    var overlap = (0, _lodash.intersection)(Object.keys(resolvers), Object.keys(propTypes));
    (0, _assert2.default)(overlap.length === 0, displayName + ' - cannot accept propTypes ' + formatList(overlap) + ' which also have resolvers');
};

var assertNoStateFromStoresPropTypesCollisions = function assertNoStateFromStoresPropTypesCollisions(displayName) {
    var state = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    var propTypes = arguments[2];

    var overlap = (0, _lodash.intersection)(Object.keys(state), Object.keys(propTypes));
    (0, _assert2.default)(overlap.length === 0, displayName + ' - cannot accept propTypes ' + formatList(overlap) + ' which are returned from stores');
};

var checkIsResolved = function checkIsResolved(resolvers, name, props) {
    var _resolvers$name = resolvers[name];
    var _resolvers$name$depen = _resolvers$name.dependencies;
    var dependencies = _resolvers$name$depen === undefined ? [] : _resolvers$name$depen;
    var isResolved = _resolvers$name.isResolved;

    return (0, _lodash.every)(dependencies, function (dependency) {
        return checkIsResolved(resolvers, dependency, props);
    }) && (isResolved ? isResolved(props) : !(0, _lodash.isUndefined)(props[name]));
};

var assertResolversNotCyclic = function assertResolversNotCyclic(displayName, resolvers) {
    var detect = function detect(path) {
        var _resolvers$last$depen = resolvers[(0, _lodash.last)(path)].dependencies;
        var dependencies = _resolvers$last$depen === undefined ? [] : _resolvers$last$depen;

        dependencies.forEach(function (dependency) {
            var cyclic = path.includes(dependency);
            (0, _assert2.default)(!cyclic, displayName + ' - circular dependency detected ' + formatList([].concat(_toConsumableArray(path.slice(path.indexOf(dependency))), [dependency])));
        });
        dependencies.forEach(function (dependency) {
            return detect([].concat(_toConsumableArray(path), [dependency]));
        });
    };
    Object.keys(resolvers).forEach(function (r) {
        return detect([r]);
    });
};

exports.default = function (stores, getStateFromStores, resolvers) {
    var propTypes = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];
    return (0, _recompose.hoistStatics)(function (C) {
        var _class, _temp;

        var displayName = (0, _recompose.wrapDisplayName)(C, 'resolver');

        assertValidResolverKeys(displayName, resolvers);
        assertValidResolverDependencies(displayName, resolvers);
        assertNoResolversPropTypesCollisions(displayName, resolvers, propTypes);
        assertResolversNotCyclic(displayName, resolvers);

        var safeGetStateFromStores = function safeGetStateFromStores(_ref3, props) {
            var _getStore = _ref3.getStore;
            var executeAction = _ref3.executeAction;

            var result = getStateFromStores({
                getStore: function getStore(store) {
                    var storeName = store.storeName || store;
                    // did we forget to add this store to the list of stores passed in?
                    // handle both strings/store objects in both the getStore calls, and stores array
                    (0, _assert2.default)(stores.find(function (s) {
                        return (s.storeName || s) === storeName;
                    }), displayName + ' - getStateFromStores should not call getStore(' + storeName + ') without adding ' + storeName + ' to the stores array');
                    return _getStore(store);
                },
                executeAction: executeAction
            }, (0, _lodash.pick)(props, Object.keys(propTypes)));
            (0, _assert2.default)((0, _lodash.isPlainObject)(result), displayName + ' - getStateFromStores should return an object rather than ' + JSON.stringify(result));
            assertNoStateFromStoresPropTypesCollisions(displayName, result, propTypes);
            return result;
        };

        return (0, _recompose.compose)((0, _recompose.setDisplayName)(displayName), (0, _fluxibleAddonsReact.connectToStores)(stores, safeGetStateFromStores, {
            executeAction: _react.PropTypes.func.isRequired
        }))((_temp = _class = function (_Component) {
            _inherits(_class, _Component);

            function _class() {
                _classCallCheck(this, _class);

                return _possibleConstructorReturn(this, Object.getPrototypeOf(_class).apply(this, arguments));
            }

            _createClass(_class, [{
                key: 'render',
                value: function render() {
                    var _this2 = this;

                    if ((0, _lodash.every)(resolvers, function (resolver, propName) {
                        return checkIsResolved(resolvers, propName, _this2.props);
                    })) {
                        return _react2.default.createElement(C, this.props);
                    }

                    var inputProps = (0, _lodash.pick)(this.props, Object.keys(propTypes));
                    var key = (0, _lodash.uniqueId)('fluxibleResolver');
                    var fluxibleResolver = (0, _reactResolver.resolve)(key, function () {
                        var initialProps = safeGetStateFromStores(_this2.context, _this2.props);
                        if ((0, _lodash.every)(resolvers, function (resolver, propName) {
                            return checkIsResolved(resolvers, propName, _extends({}, inputProps, initialProps));
                        })) {
                            return initialProps;
                        }

                        var resolverDefers = (0, _lodash.mapValues)(resolvers, function () {
                            return _q2.default.defer();
                        });
                        (0, _lodash.each)(resolverDefers, function (resolverDefer, propName) {
                            var _resolvers$propName = resolvers[propName];
                            var _resolvers$propName$d = _resolvers$propName.dependencies;
                            var dependencies = _resolvers$propName$d === undefined ? [] : _resolvers$propName$d;
                            var resolver = _resolvers$propName.resolver;


                            resolverDefer.resolve(_q2.default.all(dependencies.map(function (dependency) {
                                return resolverDefers[dependency].promise;
                            })).then(function () {
                                return safeGetStateFromStores(_this2.context, _this2.props);
                            }).then(function (props) {
                                if (checkIsResolved(resolvers, propName, _extends({}, inputProps, props))) {
                                    return _q2.default.resolve();
                                }
                                var defer = _q2.default.defer();
                                _this2.context.executeAction(function (actionContext) {
                                    defer.resolve(actionContext.executeAction(resolver, _extends({}, inputProps, props)));
                                });
                                return defer.promise.tap(function (res) {
                                    var state = safeGetStateFromStores(_this2.context, _this2.props);
                                    (0, _assert2.default)(checkIsResolved(resolvers, propName, _extends({}, inputProps, state)), displayName + ' - unfulfilled resolution for "' + propName + '" in ' + (0, _recompose.getDisplayName)(C) + ' - resolver returned ' + JSON.stringify(res) + ', getStateFromStores returned ' + JSON.stringify(state[propName]));
                                });
                            }));
                        });
                        return _q2.default.all((0, _lodash.values)(resolverDefers).map(function (_ref4) {
                            var promise = _ref4.promise;
                            return promise;
                        })).then(function () {
                            return safeGetStateFromStores(_this2.context, _this2.props);
                        });
                    });
                    var Container = (0, _recompose.compose)(fluxibleResolver, (0, _recompose.mapProps)(function (props) {
                        return (0, _lodash.omit)(props, key);
                    }))(C);
                    return _react2.default.createElement(Container, this.props);
                }
            }]);

            return _class;
        }(_react.Component), _class.contextTypes = fluxibleContextTypes, _class.propTypes = propTypes, _temp));
    });
};

module.exports = exports['default'];
//# sourceMappingURL=index.js.map